#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NKEYS 12
#define STRMAX 1024
#define COORDSYS "CoordSys Earth Projection 1, 104\n"

/* ========================================================================== *
 * swapxy
 *
 * Program to read objects from one MIF file,
 * replace CoordSys statement with one from 'swapxy.prj' or COORDSYS
 * swap x, y ,
 * and write to another MIF file.
 *
 * Usage: swapxy <input> <output>
 *
 * No actions on MID file are carried out,
 * just use original one copying or renaming.
 * -------------------------------------------------------------------------- */
int main(int argc, char *argv[])
{
  char buf[STRMAX], w[8][128];
  char *keyword[NKEYS] = {
    "Arc", "Ellipse", "Line", "Pline", "Point", "Region", "Rect",
    "Roundrect", "Text", "Multipoint", "Center", "Multiple"
  };
  char crs[STRMAX] = COORDSYS; /* beware the 255 char limit */
  double a1, a2;
  int nsec, npt, t, k;
  FILE *fp0, *fp1;

  if (argc < 3) {
    puts("usage: swapxy <input> <output>");
    exit(EXIT_SUCCESS);
  }
  if ((fp0 = fopen("swapxy.prj", "r")) != NULL) {
    if (fgets(buf, STRMAX, fp0) != NULL) {
      strcpy(crs, buf);
    }
  }
  fclose(fp0);
  if ((fp0 = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "can't open %s\n", argv[1]);
    exit(EXIT_FAILURE);
  }
  if ((fp1 = fopen(argv[2], "w")) == NULL) {
    fprintf(stderr, "can't create %s\n", argv[2]);
    exit(EXIT_FAILURE);
  }
  nsec = npt = t = 0;
  while (fgets(buf, STRMAX, fp0) != NULL) {
    if (!strncmp(buf, "CoordSys", 8)) {
      fputs (crs, fp1);
      continue;
    }
    if (npt == 0) {
      if (nsec == 0) {
	switch (t) {
	case 2: /* Text string */
	  fputs(buf, fp1);
	  t = 1;
	  break;
	case 1: /* Text bbox */
	  sscanf(buf, "%s %s %s %s", w[0], w[1], w[2], w[3]);
	  fprintf(fp1, "    %s %s %s %s\n", w[1], w[0], w[3], w[2]);
	  t = 0;
	  break;
	default:
	  sscanf(buf, "%s %s %s %s %s %s %s",
		 w[0], w[1], w[2], w[3], w[4], w[5], w[6]);
	  for (k = 0; k < NKEYS - 1; k++) {
	    if (strcmp(w[0], keyword[k]) == 0)
	      break;
	  }
	  switch (k) {
	  case 8: /* "Text" */
	    t = 2;
	    fputs(buf, fp1);
	    break;
	  case 9: /* "Multipoint" */
	    npt = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 5: /* "Region" */
	    nsec = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 3: /* "Pline" */
	    if (strcmp(w[1], keyword[NKEYS - 1]) == 0) /* "Pline Multiple" */
	      nsec = atoi(w[2]);
	    else
	      npt = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 4: /* "Point" */
	    fprintf(fp1, "%s %s %s\n", w[0], w[2], w[1]);
	    break;
	  case 10: /* "Center" */
	    fprintf(fp1, "    %s %s %s\n", w[0], w[2], w[1]);
	    break;
	  case 1: /* "Ellipse" */
	  case 2: /* "Line" */
	  case 6: /* "Rect" */
	    fprintf(fp1, "%s %s %s %s %s\n",
		    w[0], w[2], w[1], w[4], w[3]);
	    break;
	  case 7: /* "Roundrect" */
	    fprintf(fp1, "%s %s %s %s %s %s\n",
		    w[0], w[2], w[1], w[4], w[3], w[5]);
	    break;
	  case 0: /* "Arc" */
	    a1 = atof(w[5]);
	    a1 = (a1 <= 90.) ? 90. - a1 : 450. - a1;
	    a2 = atof(w[6]);
	    a2 = (a2 <= 90.) ? 90. - a2 : 450. - a2;
	    fprintf(fp1, "%s %s %s %s %s %g %g\n",
		    w[0], w[2], w[1], w[4], w[3], a2, a1);
	    break;
	  default:
	    fputs(buf, fp1);
	  }
	}
      } else {
	npt = atoi(buf);
	fputs(buf, fp1);
	nsec--;
      }
    } else {
      sscanf(buf, "%s %s", w[0], w[1]);
      fprintf(fp1, "%s %s\n", w[1], w[0]);
      npt--;
    }
  }
  fclose(fp1);
  fclose(fp0);
  return 0;
}
/* ========================================================================== */
